@@ -16,7 +16,7 @@ PLURAL = {
     ES: "es",
 }
 MORE_MESSAGES = {
-    EN: "...and {total} validation error{plural}} more",
+    EN: "...and {total} validation error{plural} more",
     ES: "...y {total} error{plural} más",
 }
 
@@ -102,7 +102,7 @@ VAL_ERROR_PARAM_IS_TOO_SMALL = {
     ES: "{name} es demasiado pequeño (mín. {minvalue})",
 }
 VAL_ERROR_PARAM_IS_TOO_BIG = {
-    EN: "{name} is too big (min. {maxvalue})",
+    EN: "{name} is too big (max. {maxvalue})",
     ES: "{name} es demasiado grande (máx. {maxvalue})",
 }
 VAL_ERROR_LIST_ITEM_HAS_INVALID_ELEMENT = {
