@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # pylint: disable=missing-docstring, C0301
+import json
 import re
 from datetime import date, time, datetime
 
@@ -75,7 +76,7 @@ class SchemaFormatError(ValueError):
 
 class SchemaValidationFailure(Exception):
 
-    def __init__(self, message: dict or str, params: dict):
+    def __init__(self, message: dict or str, params: dict or None = None):
         if isinstance(message, dict):
             self.message = message.get(lang.DEFAULT)
             if self.message is None:
@@ -169,42 +170,27 @@ class SchemaValidator:
         if value is None and defvalue is not None:
             value = defvalue
 
-        # -- comprueba si es mandatorio (fuera de comprobación de mensaje específico)
-        keyword = KEYWORD_MANDATORY
-        mandatory = self._get_bool(keyword)
-        if mandatory:
-            _check_mandatory(label, value)
-
         # -- aplica las funciones de transformación
         keyword = KEYWORD_TO
         tos = self._get_str(keyword)
-        if tos is not None:
-            tos = tos.split(',')
-            for to in tos:
-                to = util.trim(to)
-                if isinstance(value, str):
-                    value = _process_to(keyword, value, to)
-                if isinstance(value, list):
-                    for (pos, item) in enumerate(value):
-                        item = _process_to(keyword, item, to)
-                        value[pos] = item
-                if isinstance(value, dict):
-                    for mkey in value.keys():
-                        item = value.get(mkey)
-                        item = _process_to(keyword, item, to)
-                        value[mkey] = item
-        else:
+        if tos is None:
+            tos = 'trim';
+        tos = tos.split(',')
+        for to in tos:
+            to = util.trim(to)
             if isinstance(value, str):
-                value = util.trim(value)
+                value = _process_to(keyword, value, to)
             if isinstance(value, list):
                 for (pos, item) in enumerate(value):
                     if isinstance(item, str):
-                        value[pos] = util.trim(item)
+                        item = _process_to(keyword, item, to)
+                        value[pos] = item
             if isinstance(value, dict):
                 for mkey in value.keys():
                     item = value.get(mkey)
                     if isinstance(item, str):
-                        value[mkey] = util.trim(item)
+                        item = _process_to(keyword, item, to)
+                        value[mkey] = item
 
         # -- comprueba el tipo 'str'
         if isinstance(value, str):
@@ -233,6 +219,8 @@ class SchemaValidator:
                 keyword = KEYWORD_MAX_LINES
                 max_lines = self._get_int(keyword)
                 _check_max_lines(label, value, max_lines)
+            else:
+                value = None
 
         # -- comprueba el tipo 'int' o 'float'
         if isinstance(value, int) or isinstance(value, float):
@@ -274,6 +262,11 @@ class SchemaValidator:
             validator = SchemaValidator(cast)
             value = validator.validate(name, value)
 
+        # -- comprueba si es mandatorio (fuera de comprobación de mensaje específico)
+        keyword = KEYWORD_MANDATORY
+        mandatory = self._get_bool(keyword)
+        if mandatory:
+            _check_mandatory(label, value)
         return value
 
     def _get_obj(self, keyword: str):
@@ -394,8 +387,11 @@ def _get_type_int(name: str, value: any) -> int or None:
     if value is None:
         return None
     if isinstance(value, str):
+        value = util.trim(value)
+        if value == '':
+            return None
         try:
-            value = int(util.trim(value))
+            value = int(value)
         except ValueError:
             _val_error_unexpected_type(name, value, KEYWORD_TYPE_INT)
     if isinstance(value, float):
@@ -409,8 +405,11 @@ def _get_type_float(name: str, value: any) -> int or None:
     if value is None:
         return None
     if isinstance(value, str):
+        value = util.trim(value)
+        if value == '':
+            return None
         try:
-            value = float(util.trim(value))
+            value = float(value)
         except ValueError:
             _val_error_unexpected_type(name, value, KEYWORD_TYPE_FLOAT)
     if isinstance(value, int):
@@ -425,13 +424,13 @@ def _get_type_bool(name: str, value: any) -> bool or None:
         return None
     if isinstance(value, str):
         sbool = util.trim(value).lower()
+        if sbool == '':
+            return None
         if sbool != KEYWORD_BOOL_TRUE and sbool != KEYWORD_BOOL_FALSE:
             _val_error_unexpected_type(name, value, KEYWORD_TYPE_BOOL)
         value = sbool == KEYWORD_BOOL_TRUE
-    elif isinstance(value, int):
-        if value != 0 and value != 1:
-            _val_error_unexpected_type(name, value, KEYWORD_TYPE_BOOL)
-        value = value == 1
+    elif isinstance(value, int) or isinstance(value, float):
+        value = value != 0
     elif not isinstance(value, bool):
         _val_error_unexpected_type(name, value, KEYWORD_TYPE_BOOL)
     return value
@@ -446,6 +445,9 @@ def _get_type_date(name: str, value: any, date_format: str) -> date or None:
         return value.date()
     elif not isinstance(value, str):
         _val_error_unexpected_type(name, value, KEYWORD_TYPE_DATE)
+    value = util.trim(value)
+    if value == '':
+        return None
     try:
         value = datetime.strptime(value, date_format)
     except ValueError:
@@ -462,6 +464,9 @@ def _get_type_time(name: str, value: any, time_format: str) -> time or None:
         return value.time()
     elif not isinstance(value, str):
         _val_error_unexpected_type(name, value, KEYWORD_TYPE_TIME)
+    value = util.trim(value)
+    if value == '':
+        return None
     try:
         value = datetime.strptime(value, time_format)
     except ValueError:
@@ -478,6 +483,9 @@ def _get_type_datetime(name: str, value: any, datetime_format: str) -> date or N
         return datetime.combine(value, datetime.min.time())
     elif not isinstance(value, str):
         _val_error_unexpected_type(name, value, KEYWORD_TYPE_DATETIME)
+    value = util.trim(value)
+    if value == '':
+        return None
     try:
         value = datetime.strptime(value, datetime_format)
     except ValueError:
@@ -495,12 +503,26 @@ def _get_type_list(name: str, value: any, separator: str) -> list or None:
     return value
 
 
-def _get_type_dict(name: str, value: any) -> list or None:
+def _get_type_dict(name: str, value: any) -> dict or None:
     if value is None:
         return None
+    if isinstance(value, str):
+        value = util.trim(value)
+        if value == '':
+            return None
+        try:
+            value = json.loads(value)
+        except json.decoder.JSONDecodeError:
+            value = value.replace('\"', '<dquote>')
+            value = value.replace('\'', '\"')
+            value = value.replace('<dquote>', '\'')
+            try:
+                value = json.loads(value)
+            except json.decoder.JSONDecodeError:
+                _val_error_unexpected_type(name, value, KEYWORD_TYPE_DICT)
     if not isinstance(value, dict):
         _val_error_unexpected_type(name, value, KEYWORD_TYPE_DICT)
-    return [value]
+    return value
 
 
 def _process_to(keyword: str, value: any, func_name: str) -> str:
@@ -516,7 +538,7 @@ def _process_to(keyword: str, value: any, func_name: str) -> str:
 
 
 def _check_mandatory(name: str, value: any):
-    if value is None or (isinstance(value, str) and len(util.trim(value)) == 0):
+    if value is None or (isinstance(value, str) and len(value) == 0):
         raise SchemaValidationFailure(lang.VAL_ERROR_PARAM_IS_MANDATORY, {KEYWORD_NAME: name})
 
 
